==> armagetron_src_0.1.4.5.tar.gz: armagetron/src/tools/tList.h <==
/*

*************************************************************************

ArmageTron -- Just another Tron Lightcycle Game in 3D.
Copyright (C) 2000  Manuel Moos (manuel@moosnet.de)

**************************************************************************

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
***************************************************************************

*/

#ifndef ArmageTron_LIST_H
#define ArmageTron_LIST_H

#include "tArray.h"
#include "tSafePTR.h"
#include <new.h>
#include <stdlib.h>

// a usefull class of lists


template <class T> class List:public tArray<T *> {
  friend T;
  int offset;

  // Array<T *> list;
public:
  
  ~List(){
    for(int i=Len()-1;i>=0;i--)
      ((int *)operator()(i))[offset]=-1;
  }
  
  List(int size=0):tArray<T *>(size){}

  void Add(T *t,int &id){
    offset=&id-((int *)t);

    if (id<0){    // tEventQueue relies on the fact that we put t in
      id=Len();   // the last place.
      operator[](id)=t;
    }
  }


  void Remove(T *t,int &id){
    // con << "offset=" << offset << '\n';
    if (id>=0 && Len()>0){
#ifdef DEBUG
      if (id>=Len())
	tERR_ERROR_INT("Scr. list structure!");

      T *test=operator()(id);
      if (test!=t)
	tERR_ERROR_INT("Screwed list structure!");
#endif
      // the check for Len() is done, since this may be
      // called on an allready descructed list.
      T *other=operator()(Len()-1);
      operator()(id)=other;
      int &other_id=((int *)other)[offset];
      other_id=id;
      /*
#ifdef tSAFEPTR // reset the last element
      //(((tCHECKED_PTR(T) *)base())+i)->~tCHECKED_PTR(T)();
      operator()(Len()-1)=NULL;
      //operator()(Len()-1).~tCHECKED_PTR(T)();
      //new (&operator()(Len()-1)) tCHECKED_PTR(T);
#endif
      */
      SetLen(Len()-1);
    }
    id=-1;
  }
};


#endif

==> armagetron_src_0.1.4.8.tar.gz: armagetron/src/tools/tList.h <==
/*

*************************************************************************

ArmageTron -- Just another Tron Lightcycle Game in 3D.
Copyright (C) 2000  Manuel Moos (manuel@moosnet.de)

**************************************************************************

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
***************************************************************************

*/

#ifndef ArmageTron_LIST_H
#define ArmageTron_LIST_H

#include "tArray.h"
#include "tSafePTR.h"
#include <new.h>
#include <stdlib.h>

// a usefull class of lists


template <class T> class List:public tArray<T *> {
  friend T;
  int offset;

  // Array<T *> list;
public:
  
  ~List(){
    for(int i=Len()-1;i>=0;i--)
      (reinterpret_cast<int *>(operator()(i)))[offset]=-1;
  }
  
  List(int size=0):tArray<T *>(size){}

  void Add(T *t,int &id){
    offset=&id-(reinterpret_cast<int *>(t));

    if (id<0){    // tEventQueue relies on the fact that we put t in
      id=Len();   // the last place.
      operator[](id)=t;
    }
  }


  void Remove(T *t,int &id){
    // con << "offset=" << offset << '\n';
    if (id>=0 && Len()>0){
#ifdef DEBUG
      if (id>=Len())
	tERR_ERROR_INT("Scr. list structure!");

      T *test=operator()(id);
      if (test!=t)
	tERR_ERROR_INT("Screwed list structure!");
#endif
      // the check for Len() is done, since this may be
      // called on an allready descructed list.
      T *other=operator()(Len()-1);
      operator()(id)=other;
      int &other_id=(reinterpret_cast<int *>(other))[offset];
      other_id=id;
      /*
#ifdef tSAFEPTR // reset the last element
      //(((tCHECKED_PTR(T) *)base())+i)->~tCHECKED_PTR(T)();
      operator()(Len()-1)=NULL;
      //operator()(Len()-1).~tCHECKED_PTR(T)();
      //new (&operator()(Len()-1)) tCHECKED_PTR(T);
#endif
      */
      SetLen(Len()-1);
    }
    id=-1;
  }
};


#endif
